FUNCTION FLUXLFIRTOSED, DATAWAVE, DATAFLUX, tdust=tdust, SEDWAVE=SEDWAVE, REDSHIFT=REDSHIFT, LOGFIR=LOGFIR, BETA=BETA, ALPHA=ALPHA, GENERALOPACITY=GENERALOPACITY

if n_params() eq 0 then begin
    print, 'fluxlfirtosed.pro'
    print, '         USAGE: sedflux = fluxlfirtosed(datawave,dataflux,redshift=redshift,[logfir=logfir,tdust=tdust,sedwave=sedwave,beta=beta,alpha=alpha,\generalopacity])'
    print, '                INPUT: datawave, dataflux, redshift, [logfir]'
    print, '                OUTPUT: sedwave, tdust'
    return, -1
endif

if keyword_set(logfir) ne 0 then begin
;the integrated S_nu function is this:
    logintS = logfir - alog10(4.*!pi) - 2.*alog10(lumdist(redshift,/cm,/silent,h0=71,lambda0=0.73)) + alog10(3.8) + 33.
endif else begin
    print, 'YOU MUST ENTER LOGFIR TO PROCEED.'
    return, -1
endelse


if keyword_set(beta) ne 0 then begin
    beta = beta
endif else begin
    beta = 1.5
endelse

if keyword_set(alpha) ne 0 then begin
   alpha = alpha
endif else begin 
   alpha = 2.0
endelse

tdarr = 2.+findgen(1500)*0.1 ;temp ranging from 2 to 152K

wavearr = 10^makehx(0.5,5.4,0.001)
nuarr = 3.e8/(wavearr*10^(-6.)) ;frequency
nubin = nuarr[1:*] - nuarr[0:n_elements(nuarr)-1]
nubin = [nubin,-1] & nubin[n_elements(nuarr)-1] = nubin[n_elements(nuarr)-2]

hoverk=alog10(6.626)-34 - alog10(1.381)+23. ; h (planck const) divided by k (boltzmann const)
arg = dblarr(n_elements(tdarr),n_elements(wavearr))
logS_nu = dblarr(n_elements(tdarr),n_elements(wavearr))
for i=0, n_elements(tdarr) - 1 do arg[i,*] = (exp((10^hoverk*nuarr)/(tdarr[i]))-1.)^(-1.)

if keyword_set(generalopacity) eq 0 then begin 
for i=0, n_elements(tdarr) - 1 do logS_nu[i,*] = (3.+beta)*alog10(nuarr) + alog10(arg[i,*])
endif else begin 
   nu0 = 3.e8/(200. * 1.e-6)
   tau = (nuarr/nu0)^beta
   for i=0, n_elements(tdarr) - 1 do logS_nu[i,*] = alog10(1-exp(-tau)) + 3.*alog10(nuarr) +  alog10(arg[i,*])
endelse

;powerlaw component:
logs_nu_pl = logs_nu*0.
for i=0, n_elements(tdarr) - 1 do begin
   tmp = max(logs_nu[i,*],imax)
   tx = alog10(wavearr[imax])
   ty = logs_nu[i,imax]
   if keyword_set(generalopacity) eq 0 then begin
   p0 = ty + alog10( (exp(14393.9/(10^tx*tdarr[i]))-1)) + (3+beta)*tx 
   critwave = (1./tdarr[i])*(0.000310933 + 6.99836e-05*alpha)^(-1.)
   offset = p0 + (3 + beta)*alog10(tdarr[i] * (0.000310933 + 6.99836e-05*alpha)) - alog10(exp(14393.9 * (0.000310933 + 6.99836e-05 * alpha ))-1)
   logS_nu_pl[i,*] = alog10(exp(-(wavearr/(critwave*0.75))^2.)) + alpha*alog10(wavearr/critwave) + offset
   endif else begin 
   p0 = ty + alog10( (exp(14393.9/(10^tx*tdarr[i]))-1)) - alog10(1-exp(-(200./10^tx)^beta)) + 3.*tx
   critwave = ((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*tdarr[i])^(-1.)
   offset = p0 + alog10(1-exp(-(200.*((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*tdarr[i]))^beta)) + 3.*alog10((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*tdarr[i]) - alog10(exp(14393.9*((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*tdarr[i])/(tdarr[i]))-1.)
   logS_nu_pl[i,*] = alog10(exp(-(wavearr/(critwave*0.75))^2.)) + alpha*alog10(wavearr/critwave) + offset
   endelse
endfor

const2 = alog10(2.*6.626e-34) - alog10(3.^2.)-8.*2.
logS_nu = logS_nu + const2
logS_nu_pl = logS_nu_pl + const2
;(log)S_nu has units of W m^-2.  Convert to erg cm^-2 s^-1 --> 10^7 for energy, (100.)^-2=10^-4 for distance
logS_nu = logS_nu + 7. - 4.
logS_nu_pl = logS_nu_pl + 7. - 4.
;(log)S_nu now has units of erg cm^-2 s^-1
S_nu = 10^logS_nu
S_nu_pl = 10^logS_nu_pl
S_nu_tot = S_nu+S_nu_pl

logS_nu_tot = alog10(S_nu+S_nu_pl)

bandwidth = where(wavearr ge 8. and wavearr le 1000.) ;integrate lfir over these wavelengths

scaler = fltarr(n_elements(tdarr))
logfirarr = scaler*0.

ls_nu = logs_nu*0.
for i=0, n_elements(tdarr) - 1 do begin
    logfirarr[i]   = alog10(-total(reform(s_nu_tot[i,bandwidth])*nubin[bandwidth]/(1+redshift))) + $
      alog10(4.*!pi)+2.*alog10(lumdist(redshift,/cm,/silent,h0=71,lambda0=0.73))-alog10(3.8)-33.-26.
    scaler[i] = logfir - logfirarr[i]
    ls_nu[i,*] = logs_nu_tot[i,*]+scaler[i]
endfor

; first find the corresponding flux values at those wavelengths for each S_nu (Td)
expectedS_nu = fltarr(n_elements(tdarr))
tmp = min(abs(alog10(wavearr)-(alog10(datawave/(1.+redshift)))[0]),tt)
expectedS_nu = reform(lS_nu[*,tt])
;help, (dataflux)[0], dataflux, expecteds_nu
;chisquared = ((alog10(dataflux))[0]-expecteds_nu)^2.  ;;; in the case when you have more than one datapoint
dff = alog10(dataflux[0])
if dff le -1. then dff = alog10(2.) ;set to 2mJy
chisquared = abs(dff-expecteds_nu) ;; in the case when you only have one datapoint

tmp0 = where(finite(chisquared),ct)
if ct gt 2 then begin
tmp2 = min(chisquared[tmp0],imin)
index1 = takeaway(findgen(n_elements(tdarr)),(findgen(n_elements(tdarr)))[tmp0[(imin-5):(imin+5)]])
tmp3 = min(chisquared[index1],imin2)
   best_td = [tdarr[imin],tdarr[imin2]]
dusttemp = min(best_td)
testindex = array_equal([dusttemp],[tdarr[imin]])
if testindex eq 0 then imin = imin2
endif else begin
   return, -1
endelse


sedwave = alog10(wavearr)
sedflux = reform(ls_nu[imin,*])
radioflux = lfirtoradio(logfir,redshift)
radio = sedwave*0.8+normalizepoint(alog10(214286./(1.+redshift)),alog10(radioflux)-3.,sedwave,sedwave*0.8)
sedflux = alog10(10^sedflux+10^radio)

tmp = max(sedflux,imax)
tdust = 2.898e3/10^sedwave[imax]

;if posderiv eq 1 then begin
;    print,'WARNING: Fit cannot be done for this lfir - do not trust the dust temperature computation.'
;    lfirnew = sedtolfir(sedwave,sedflux,8.,1000.,redshift=redshift)
;    print,'WARNING: LFIR OF FIT = ', 10^lfirnew, ' Lsun'
;endif

;stop
return,sedflux

END
