FUNCTION FLUXTDTOSED, DATAWAVE, DATAFLUX, TD=TD, SEDWAVE=SEDWAVE, REDSHIFT=REDSHIFT, LOGFIR=LOGFIR, BETA=BETA, ALPHA=ALPHA, NORADIO=NORADIO, GENERALOPACITY=GENERALOPACITY

if n_params() eq 0 then begin
    print, 'fluxtdtosed.pro'
    print, '         USAGE: sedflux = fluxtdtosed(datawave,dataflux,redshift=redshift,[Td=Td,sedwave=sedwave,logfir=logfir,beta=beta,alpha=alpha,/noradio])'
    print, '                INPUT: datawave, dataflux, [td]'
    print, '                OUTPUT: sedwave, [logfir]'
    print, ''
    print, ''
    return, -1
endif

if keyword_set(td) eq 0 then td=30

;if keyword_set(logfir) ne 0 then begin
;t;he integrated S_nu function is this:
    ;logintS = logfir - alog10(4.*!pi) - 2.*alog10(lumdist(redshift,/cm,h0=71,lambda0=0.73,/silent)) + alog10(3.8) + 33.
;endif

if keyword_set(beta) ne 0 then begin
    beta = beta
endif else begin
    beta = 1.5
endelse

if keyword_set(alpha) ne 0 then begin
   alpha = alpha
endif else begin 
   alpha = 2.0
endelse

wavearr = 10^makehx(0.5,5.4,0.001)
nuarr = 3.e8/(wavearr*10^(-6.)) ;frequency
nubin = nuarr[1:*] - nuarr[0:n_elements(nuarr)-1]
nubin = [nubin,-1] & nubin[n_elements(nuarr)-1] = nubin[n_elements(nuarr)-2]

hoverk=alog10(6.626)-34 - alog10(1.381)+23. ; h (planck const) divided by k (boltzmann const)
arg = dblarr(n_elements(wavearr))
logS_nu = dblarr(n_elements(wavearr))
arg = (exp((10^hoverk*nuarr)/(td))-1.)^(-1.)

if keyword_set(generalopacity) eq 0 then begin 
   logS_nu = (3.+beta)*alog10(nuarr) + alog10(arg)
endif else begin 
   nu0 = 3.e8/(200. * 1.e-6)
   tau = (nuarr/nu0)^beta
   logS_nu = alog10(1-exp(-tau)) + 3.*alog10(nuarr) +  alog10(arg)
endelse

;powerlaw component:
tmp = max(logs_nu,imax)
tx = alog10(wavearr[imax])
ty = logs_nu[imax]
if keyword_set(generalopacity) eq 0 then begin
   p0 = ty + alog10( (exp(14393.9/(10^tx*td))-1)) + (3+beta)*tx 
                                ;optically thin:
   critwave = (1./td)*(0.000310933 + 6.99836e-05*alpha)^(-1.)
   offset = p0 + (3 + beta)*alog10(td * (0.000310933 + 6.99836e-05*alpha)) - alog10(exp(14393.9 * (0.000310933 + 6.99836e-05 * alpha ))-1)
endif else begin 
                                ;general opacity:
   p0 = ty + alog10( (exp(14393.9/(10^tx*td))-1)) - alog10(1-exp(-(200./10^tx)^beta)) + 3.*tx
   critwave = ((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td)^(-1.)
   offset = p0 + alog10(1-exp(-(200.*((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td))^beta)) + 3.*alog10((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td) - alog10(exp(14393.9*((26.6764 + 6.24629 * alpha)^(-2.) + (0.000190530 + 7.24277e-05 * alpha)*td)/(td))-1.)
endelse
;   critwave = 1./(5.08308e-6 + 0.000927256 * td)

logS_nu_pl = alog10(exp(-(wavearr/(critwave*0.75))^2.)) + alpha*alog10(wavearr/critwave) + offset


   const2 = alog10(2.*6.626e-34) - alog10(3.^2.)-8.*2.
   logS_nu = logS_nu + const2
   logS_nu_pl = logS_nu_pl + const2
;(log)S_nu has units of W m^-2.  Convert to erg cm^-2 s^-1 --> 10^7 for energy, (100.)^-2=10^-4 for distance
   logS_nu = logS_nu + 7. - 4.
   logS_nu_pl = logS_nu_pl + 7. - 4.
;(log)S_nu now has units of erg cm^-2 s^-1
   S_nu = 10^logS_nu
   S_nu_pl = 10^logS_nu_pl

logS_nu_tot = alog10(S_nu+S_nu_pl)


bandwidth = where(wavearr ge 8. and wavearr le 1000.) ;integrate lfir over these wavelengths

intS = alog10(-total(reform(S_nu[bandwidth])*nubin[bandwidth]/(1+redshift))) ;/nuarr[bandwidth]))
sedwave = alog10(wavearr)
sedflux = logs_nu_tot - intS+26. + normalizepoint(alog10(datawave/(redshift+1.)),alog10(dataflux),sedwave,logs_nu_tot-intS+26.)
logintS = normalizepoint(alog10(datawave/(redshift+1.)),alog10(dataflux),sedwave,logs_nu_tot-intS+26.)
logfir = logintS + alog10(4.*!pi) + 2.*alog10(lumdist(redshift,/cm,h0=71,lambda0=0.73,/silent)) - alog10(3.8) - 33.
radioflux = lfirtoradio(logfir,redshift)
if keyword_set(noradio) eq 0 then begin
    radio = sedwave*0.8+normalizepoint(alog10(214286./(1.+redshift)),alog10(radioflux)-3.,sedwave,sedwave*0.8)
    sedflux = alog10(10^sedflux+10^radio)
endif

return,sedflux

END
