FUNCTION SEDDETLIM, LIMWAVE, LIMFLUX, REDSHIFT, TDARR=TDARR

if n_params() eq 0 then begin
   print, "    SEDDETLIM.PRO - computes the luminosity detection limit as a function of Td. given a wavelength/flux limit and redshift"
   print, "            usage - lumfn = seddetlim(limitwavelength, limitfluxdensity, redshift, tdarr=tdarr)"
   print, "                        input: limitwavelength - the wavelength of the detection limit in the FIR"
   print, "                               limitfluxdensity - the detection limit @limitwavelength in mJy"
   print, "                               redshift - desired redshift of limit function"
   print, "                       output: tdarr - the dust temperature array, which you can use to plot on lfir/td as: djs_oplot, lumfn, tdarr ... "
   print, "    cmc - august 2011"
   return, -1
endif

; NOTE - must make this a function of Td since it's degenerate with luminosity

tdarr = findgen(400)*0.5+2.
;tdarr ranges from 2K - 202K

tdarrtmp = tdarr*0.
lfirarr = tdarr*0.
for i=0, n_elements(tdarr) - 1 do begin 
   sf = fluxtdtosed(limwave,limflux,redshift=redshift,td=tdarr[i],logfir=logfirtmp,sedwave=sw)
   lfirarr[i] = logfirtmp
   tmp = max(sf,imax)
   tdarrtmp[i] = 2.898e3/(10^sw[imax])
endfor

tdarr = tdarrtmp


return, lfirarr


END
