FUNCTION SEDDETLIM_td, LIMWAVE, LIMFLUX, TD=TD, ZARR=ZARR

if n_params() eq 0 then begin
   print, "    SEDDETLIM_TD.PRO - computes the luminosity detection limit as a function of z. given a wavelength/flux limit and redshift array"
   print, "            usage - lumfn = seddetlim_TD(limitwavelength, limitfluxdensity, zarr=zarr, td=td)"
   print, "                        input: limitwavelength - the wavelength of the detection limit in the FIR"
   print, "                               limitfluxdensity - the detection limit @limitwavelength in mJy"
   print, "                               td - assumed dust temperature"
   print, "                               zarr - the redshift array"
   print, "    cmc - april 2012"
   return, -1
endif

; NOTE - must make this a function of Td since it's degenerate with luminosity

tdust = td
lfirarr = fltarr(n_elements(zarr))
sfr = tdtosed(tdust,redshift=zarr[1],logfir=12.,sedwave=swr,/generalopacity)

for j=0, n_elements(zarr) - 1 do begin 
   offset = normalizepoint(alog10(limwave)-alog10(1+zarr[j]),alog10(limflux),swr,sfr) 
   lfirarr[j] = sedtolfir(swr,sfr+offset,8.,1000,redshift=zarr[j]) 
   endfor

return, lfirarr

END
