FUNCTION SEDTOLFIR, SEDWAVE, SEDFLUX, INTLOW, INTHI, redshift=redshift

if n_params() eq 0 then begin
    print, ' sedtolfir.pro'
    print, '         USAGE: result = sedtolfir(sedwave,sedflux,intlow,inthigh,redshift=redshift)'
    print, '                 sedwave - wavelength array, in log(wave[um])'
    print, '                 sedflux - flux array, in log(flux[mJy])'
    print, '                    NOTE: sedwave/sedflux is REST-FRAME, redshift is just used to calculate luminosity via dL'
    print, '                 intlow,inthigh - integration limits for calculating L_FIR (e.g. 8um-1000um)'
    print, '                 redshift - you must input redshift'
    print, '                   result in units of log solar luminosities'
    print, '         cmc - 13 mar 2009'
    return, -1
endif

wavearr = 10^sedwave

nuarr = 3.e8/(wavearr*10^(-6.)) ;frequency
nubin = nuarr[1:*] - nuarr[0:n_elements(nuarr)-1]
nubin = [nubin,-1] & nubin[n_elements(nuarr)-1] = nubin[n_elements(nuarr)-2]
beta = 1.5 ;beta of the greybody

bandwidth = where(wavearr ge intlow and wavearr le inthi) ;integrate lfir over these wavelengths

logs_nu = sedflux - 26.
s_nu = 10^logs_nu

intseff = alog10(-total(s_nu[bandwidth]*nubin[bandwidth]/(1+redshift)))
logfir = intseff + alog10(4.*!pi) + 2.*alog10(lumdist(redshift,/cm,/silent,h0=71,lambda0=0.73)) - alog10(3.8)-33.

; in solar luminosities
RETURN, logfir

END
