FUNCTION SEDTOsir, SEDWAVE, SEDFLUX, INTLOW, INTHI, REDSHIFT=redshift

if n_params() eq 0 then begin
    print, ' sedtosir.pro'
    print, '         USAGE: result = sedtosir(sedwave,sedflux,intlow,inthigh,redshift=redshift)'
    print, '                 sedwave - wavelength array, in log(wave[um])'
    print, '                 sedflux - flux array, in log(flux[mJy])'
    print, '                    NOTE: sedwave/sedflux is REST-FRAME, ALSO result is in units of W/m^2'
    print, '                 intlow,inthigh - integration limits for calculating S_IR (e.g. 8um-1000um)'
    print, '                 (redshift is needed for proper bandwidth contraction of sedflux)'
    print, '         cmc - 27 jan 2012'
    return, -1
endif

wavearr = 10^sedwave

nuarr = 3.e8/(wavearr*1.e-06) ;frequency
nubin = nuarr[1:*] - nuarr[0:n_elements(nuarr)-1]
nubin = [nubin,-1]
nubin[n_elements(nuarr)-1] = nubin[n_elements(nuarr)-2] ;replace the last element

bandwidth = where(wavearr ge intlow and wavearr le inthi) ;integrate lfir over these wavelengths

logs_nu = sedflux - 29. ;s_nu in units of W m^-2 Hz^-1
s_nu = 10^logs_nu

intseff = alog10(-total(s_nu[bandwidth]*nubin[bandwidth]/(1+redshift)))

; in units of W m^-2
RETURN, intseff

END
